<?php
abstract class DAODomainClass extends DomainClass
{
	protected $changedProperties = array();

// to cast boolean: filter_var( $someValue, FILTER_VALIDATE_BOOLEAN);

	function __construct( $classContext) {
		 parent::__construct( $classContext);
	}

	abstract public function getDAOName();		//eg: 'UserDAO'
	abstract public function getIDProperty();	//eg: 'usr_id'


	public function load( $entityId) {
		$db = loadModule('Database', $this->classContext);
		$userDAO = $db->getDAO( $this->getDAOName() );
		$this->entityData = $userDAO->retrieve( $entityId);
		$this->changedProperties = array();	//reset
		return $this->isLoaded();
	}

	public function isLoaded() {
		return !empty( $this->entityData);
	}
	public function reload() {
		return $this->load( $this->entityData[ $this->getIDProperty() ] );
	}
	
	
	public function getID() {
		return $this->get( $this->getIDProperty() );
	}
	
	//returns true on apparent success - if $autoCreate = true, a new entity is created if no id key present or value is empty.
	//returns false also if nothing has changed.
	public function save( $autoCreate = true) {
		
		if( empty( $this->changedProperties) ) {
			//nothing to do - lets trust this!
			return false;
		}
		
		$db = loadModule('Database', $this->classContext);
		$userDAO = $db->getDAO( $this->getDAOName());

		$saveSuccess = true;
		
		//attempt to update:
		if( array_key_exists( $this->getIDProperty(), $this->entityData) && !empty( $this->entityData[ $this->getIDProperty()] ) ) {
			//we have a non-null id - attempt to update:
			//$affecrtedRowCount = $userDAO->update( $this->entityData[ $this->getIDProperty()], $this->entityData);
			$affectedRowCount = $userDAO->update( $this->entityData[ $this->getIDProperty()], $this->changedProperties);
			if( $affectedRowCount < 1) {
				$saveSuccess = false;
			}

		} else {
			//we do not have an object id, so:
			if( $autoCreate) {
				$newEntityId = $userDAO->create( $this->prepareEntityCreateData( $this->changedProperties) );
				if( empty( $newEntityId) ) {
					$saveSuccess = false;
				} else {
					$this->entityData[ $this->getIDProperty() ] = $newEntityId;
				}
			} else {
				$saveSuccess = false;
			}

		}

		//SHOULD NOT DO THIS AS GENERAL CASE - or maybe???
		if( $saveSuccess) {
			//necesary - only in the case that a DAO 'update' or 'retrieve' function performs adhoc handling.
			$this->reload();
		}
		
		return $saveSuccess;
	}

	//useful for integration with existing code:
	public function loadFromData( $entityData) {
		$this->entityData = $entityData;
		$this->changedProperties = $this->entityData;
		return $this->isLoaded();
	}
	
	
	//call the private delete function to delete underlying DB data, then unset this User object
	//as it has no purppose.
	public static function delete( &$userObj) {
		$deleteSuccess = $userObj->deleteEntity();
		$userObj = null;
		return $deleteSuccess;
	}

	//return true on delete success, false otherwise
	private function deleteEntity() {
		$db = loadModule('Database', $this->classContext);
		$userDAO = $db->getDAO( $this->getDAOName());
		return $userDAO->delete( $this->entityData[ $this->getIDProperty()] );
	}

	public function set( $property, $value, $andSave = false) {
		
		if( !array_key_exists( $property, $this->entityData) ) {
			//its a new property:
			$this->changedProperties[ $property] = $value;
		} else {
			//check if value is different from current value:
			if( $this->entityData[ $property] != $value) {
				$this->changedProperties[ $property] = $value;
			}
		}

		$this->entityData[ $property] = $value;
		if( $andSave) {
			$this->save();
		}
	}


	protected function prepareEntityCreateData( $entityData) {
		return $entityData;
	}

}
?>