<?php
class WorkerCacheDAO extends DAO {
	
	function WorkerCacheDAO() {
	}


	function setAssigned( $workerId, $jobData = '{}') {
		return $this->update( $workerId, array('wrkr_in_use' => 1, 'wrkr_job_data' => $jobData) );
	}
	
	//returns true if job has finised (all workers are available)
	function setAvailable( $workerId, $jobData = '{}') {
		return $this->update( $workerId, array('wrkr_in_use' => 0, 'wrkr_job_data' => $jobData) );
	}
	
	//returns true if job has finised (all workers are available)
	function getInUseCount() {
		return $this->recordCountForValue( 'worker_cache', 'wrkr_in_use', 1);
	}
	
	//returns first available worker AND sets it as taken
	// or FALSE if none available.
	// or FALSE if timeout (if wait specified)
	function getWorker( $waitUntilAvailable, $waitTimeoutSecs = 60) {
		//should return 1 or 0 results.
		$statement = $this->connection()->Prepare('SELECT * FROM worker_cache WHERE wrkr_in_use = 0 LIMIT 1');
		$resultSet = $this->executePreparedStatement( $statement, array() );
		
		if( false != $resultSet->fields) {

			$this->setAssigned( $resultSet->fields['wrkr_id']);
			return $resultSet->fields;
		}

		if( $waitUntilAvailable && false === $resultSet->fields) {
			//logMessage('waiting for a free worker...');
			$sleepDuration = 2;
			$scriptTimeExtension = $sleepDuration + 20;
			$secondsWaited = 0;
			while( $secondsWaited < $waitTimeoutSecs) {
				//extend script execution time:
				set_time_limit( $scriptTimeExtension);

				sleep( $sleepDuration); //wait a second...
				$secondsWaited++;
				//logMessage('seconds waiting: ' . $secondsWaited);

				$worker = $this->getWorker( false);
				if( $worker != false) {
					//great, we've got one:
					//logMessage('great, found an available worker!');
					return $worker;
				}
			}

			logMessage('EEK! getWorker timed out.');
		}
		
		return false;
	}

	//GENERIC UPDATE FUNCTION (migrate other update functions to use this one.)
	function update( $workerId, $updateRecord) {
		$this->connection()->AutoExecute('worker_cache', $updateRecord, 'UPDATE', "wrkr_id = '$workerId'");
		$affectedRowCount = $this->connection()->Affected_Rows();
		return $affectedRowCount;
	}
	
	//GENERIC UPDATE FUNCTION (migrate other update functions to use this one.)
	function resetAllWorkers() {
		$updateRecord = array(
			'wrkr_in_use' => 0,
			'wrkr_job_data' => '{}',
		);
		$this->connection()->AutoExecute('worker_cache', $updateRecord, 'UPDATE', true);
		$affectedRowCount = $this->connection()->Affected_Rows();
		return $affectedRowCount;
	}

	function create( $dataRecord) {
		$result = $this->connection()->AutoExecute('worker_cache', $dataRecord, 'INSERT');
		$newRecordId = $this->connection()->Insert_ID();
		$affectedRowCount = $this->connection()->Affected_Rows();
		return $newRecordId;
	}
	
	//returns null if user not found.
	function retrieve( $workerId) {
		
		//should return 1 or 0 results.
		$statement = $this->connection()->Prepare('SELECT * FROM worker_cache WHERE wrkr_id = ?');
		$resultSet = $this->executePreparedStatement( $statement, array( $workerId) );
		
		if( false == $resultSet->fields) {
			//no records brought back...
			return null;
		}
		return $resultSet->fields;
	}
	
	function retrieveAll() {

		$statement = $this->connection()->Prepare('SELECT * FROM worker_cache');
		$resultSet = $this->executePreparedStatement( $statement, array() );
		
		$resultArray = array();
		
		if( false == $resultSet->fields) {
			//no records brought back...
			return $resultArray;
		}
		
		if( !$resultSet){
			//print $this->connection()->ErrorMsg();
			return $resultArray;
		} else {
			while (!$resultSet->EOF) {
				array_push( $resultArray, $resultSet->fields);
				$resultSet->MoveNext();
			}
		}
		
		return $resultArray;
	}
	
	function delete( $workerId) {
		$statement = $this->connection()->Prepare('DELETE FROM worker_cache WHERE wrkr_id = ?');
		$resultSet = $this->executePreparedStatement( $statement, array( $workerId));
		$affectedRows = $this->connection()->Affected_Rows();
		if( 1 == $affectedRows) {
			return true;										
		}
		return false;
	}
}
?>